@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-from mock import Mock
+from unittest.mock import Mock
+
 import pytest
 
 from pyee import EventEmitter
