@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 from collections import OrderedDict
+from pickle import dumps, loads
+from unittest.mock import Mock
 
-from mock import Mock
 from pytest import raises
 
 from pyee import EventEmitter
@@ -278,3 +279,11 @@ def test_properties_preserved():
     # Calling the event handler directly doesn't clear the handler
     ee.emit("once")
     call_me_also.assert_called_once()
+
+
+def test_if_serializable():
+    """`pickle`ing should not throw."""
+    ee = EventEmitter()
+    ee_copy = loads(dumps(ee))
+    assert ee._lock
+    assert ee_copy._lock
